-----------------------------
-- Variables
-----------------------------

-----------------------------
-- Unsaved Variables
-----------------------------

-- Is the frame locked in place
local GRTLocked = true;

-- Current list of skills to be displayed
local GRT_CurrentSkill = {};
-- Number of skills in GRT_CurrentSkill
local GRT_SkillCount = 0;

-- Value of last energy increase (usually 20)
local GRT_LastEnergy = 20;

-- stored energy from the last time the energy changed
local GRT_CurrEnergy = 100;

-- Total number of possible skills (not used atm)
local GRT_TickerCount = 11;

-- Total number of buttons defined in the xml file
local GRT_ButtonCount = 12;

-- When true, messages sent to GRT_Debug are displayed in chat channel
local GRT_Debugging = false;

-- Used to check whether player is entering game for the first time or not
local GRT_EnteredGame = false;

-- Stores total skills available
local GRT_TotalSkillCount = 12;

-- Used for cooldown checking
local GRT_Timer = {};


-- From BFC
-- Text Display Types, i removed HotKey because its ugly and doesnt work on all bars
GRT_DisplayTypes = {
	"Count", "Name",
};
GRT_Type = 1;
-----------------------------


-----------------------------
-- Fixed
-----------------------------
-- Stores total skills available
local GRT_TOTALSKILLCOUNT = 12;

-- Version info
local GRT_VERSION = 2.00;
local GRT_VERSION_DATE = "06/12/06";

-- Height of each button
local GRT_BUTTON_HEIGHT = 19;
-----------------------------


-----------------------------
-- Saved Variables
-----------------------------
-- GRT is addon specific, stored under Gendi. incase I should release more addons and to try and to try to keep storage tidy
Gendi = {};
Gendi.GRT = {};
Gendi.GRT.Visible = true;
Gendi.GRT.Scale = 1.0;
Gendi.GRT.Locked = false;
Gendi.GRT.FullSkillList = {};
Gendi.GRT.Width = 105.0;
Gendi.GRT.Version = 0.01;
Gendi.GRT.icons = false;
Gendi.GRT.window = true;

-----------------------------
-----------------------------


-----------------------------
-- Functions
-----------------------------

-----------------------------
-- All debug message use this function to allow easy disabling
-----------------------------
function GRT_Debug(msg)
	if ( GRT_Debugging or force) then
		-- Debug messages in red
		DEFAULT_CHAT_FRAME:AddMessage(msg, 1, 0, 0);
	end
end


-----------------------------
-- Send message to chat window (in yellow)
-----------------------------
function GRT_Chat(msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg, 1, 1, 0);
end


-----------------------------
-- split a string by the first space found, or return original string and blank string
-----------------------------
function GRT_SplitSpace(strtosplit)
	if (strtosplit) then
		local str1 = strtosplit;
	        local str2 = "";
	        local idx = strfind(strtosplit, " ");

		if ( idx ) then
	                str1 = strsub(strtosplit, 1, idx-1);
	                str2 = strsub(strtosplit, idx+1);
	        end

	        return str1, str2;
	else
		return "", "";
	end
end

-----------------------------
-- Take the default skill list and configure which lists should be displayed.
-- Then setup initial display
-----------------------------
function GRT_SetupSkillDisplayList()
	
	GRT_Debug("Starting Skill list setup");
	
	-- Set current skill list count to zero
	GRT_SkillCount = 0;
	GRT_CurrentSkill = {};
	
	-- Work through entire spellbook
	local i = 1
	while true do
		-- 1.7 remove
		local spellName, spellRank = GetSpellName(i, "spell")
		GRT_Debug(spellName);
		GRT_Debug(spellRank);
		-- 1.7 local spellName, _ = GetSpellName(i, "spell")
		if not spellName then
			-- No more spells, do loop
			do break end
		end
		
		GRT_Debug("Processing saved list");
		-- Loop through each skill in default skill list found in localization.lua
		local j = 1
		while Gendi.GRT.FullSkillList[j] do
			local value = Gendi.GRT.FullSkillList[j];
		--for key, value in Gendi.GRT.FullSkillList do
			-- If skill matches current spellName in spellbook then update skill based on spell info
			if (spellName == value.spellName and value.enabled) then
				GRT_Debug(value.spellName.." matches spellbook entry");
				-- Increase skill count
				GRT_SkillCount = GRT_SkillCount + 1;
				-- Set next GRT_CurrentSkill to all entries in default list for this spell
				GRT_CurrentSkill[GRT_SkillCount] = {};
				GRT_CurrentSkill[GRT_SkillCount].spellName = value.spellName;
				GRT_CurrentSkill[GRT_SkillCount].energyDefault = value.energyDefault;
				GRT_CurrentSkill[GRT_SkillCount].stype = value.stype;
				GRT_CurrentSkill[GRT_SkillCount].enabled = value.enabled;
				GRT_CurrentSkill[GRT_SkillCount].Cooldown = 0;
				
				-- Set skill spellId (unique to player)
				GRT_CurrentSkill[GRT_SkillCount].spellId = i;
				-- Set skill rank (unique to player)
				-- 1.7 remove
				GRT_CurrentSkill[GRT_SkillCount].rank = spellRank;
				
				-- Set energy used per skill use
				GRT_CurrentSkill[GRT_SkillCount].energy = GRT_CurrentSkill[GRT_SkillCount].energyDefault;
				-- Get texture based of spellId
				GRT_CurrentSkill[GRT_SkillCount].texture = GetSpellTexture(i, "spell");
				-- As energy will very likely be 100 at this point, set skill as ready
				GRT_CurrentSkill[GRT_SkillCount].ready = true;
				
				-- Update energy cost based on extra talents
				-- If skill is sinister stike
				--[[
				if (SpellName == GRT_SINISTERSTRIKE) then
					local _, _, _, _, currRank, _= GetTalentInfo( 2, 2);
					-- if player as talent to reduce sinister strike cost
					if ( currRank == 1) then
						value.energy = value.energyDefault -3;
					elseif ( currRank == 2) then
						value.energy = value.energyDefault -5;
					end
				end
				]]
			end
			j = j + 1;
		end
		--GRT_Debug("Saved list processed");
		
		-- Move on to next spellbook entry
		i = i + 1
	end
	-- end spellbook loop
	
	-- Sort skills (sorts first by type, finishing or instant and then alphabetically)
	GRT_SortSkills( GRT_CurrentSkill, GRT_SkillCount);
	
	-- As this can only be done after type/alpha sort, we loop through currentskill list again here
	local j = 1;
	while GRT_CurrentSkill[j] do
		local value = GRT_CurrentSkill[j];
		local key = j;
	--for key, value in GRT_CurrentSkill do
		-- Set texture, decription and status for button of same button number as skill place in list.
		-- => skills are displayed on screen in the same order as they are sorted and stored in GRT_CurrentSkill
		getglobal("GRTButton"..key.."Icon"):SetTexture(value.texture);
		getglobal("GRTButton"..key.."DescribeText"):SetText(value.spellName);
		getglobal("GRTTicker"..key.."TickerText"):SetText("READY");
		
		local Button = getglobal("GRTButton"..key);
		local Ticker = getglobal("GRTTicker"..key);
		
		Button:SetAttribute("spell",GRT_CurrentSkill[j].spellName);
		Ticker:SetAttribute("spell",GRT_CurrentSkill[j].spellName);
				
		-- If skill is a finishing move then do not highlight.
		-- Later finishing moves are highlighted in increments as cooldown points increase
		if ( value.stype == "finish") then
			Button:SetBackdropColor(0, 1, 0, 0);
			Ticker:SetBackdropColor(0, 1, 0, 0);
		else
			Button:SetBackdropColor(0, 1, 0, 0.5);
			Ticker:SetBackdropColor(0, 1, 0, 0.5);
		end
		Button:SetWidth(Gendi.GRT.Width);
		Button:Show();
		Ticker:Show();
		j=j+1;
	end
	-- end display setup loop
	
	-- Loop through the number of buttons and hide those without a skill
	local x = 1;
	while x <= GRT_ButtonCount do
		local button = getglobal("GRTButton"..x);
		local ticker = getglobal("GRTTicker"..x);
		local skillInfo = GRT_CurrentSkill[button:GetID()];
		if (skillInfo == nil or not skillInfo.enabled) then
			if (button:IsVisible()) then
				button:Hide();
			end
			if (ticker:IsVisible()) then
				ticker:Hide();
			end
		end
		-- Set button timer to zero
		GRT_Timer[x] = 0;
		x = x + 1;
	end
	-- end hiding loop
	
	-- Set the height of the main frame based on the number of visible buttons
	local height = 10 + (GRT_BUTTON_HEIGHT*GRT_SkillCount);
	getglobal("GRTFrame"):SetHeight(height);
	getglobal("GRTFrame"):SetScale(Gendi.GRT.Scale);
end


-----------------------------
-- Command prompt interface (called with /grt)
-----------------------------
function GRTCommand(argument)

	-- Feedback usage syntax if no arguments sent
	if ( not argument or argument == "" ) then
		GRT_Chat(
			GRT_USAGE1.."\n"..
			GRT_USAGE2.."\n"..
			GRT_USAGE3.."\n"..
			GRT_USAGE4.."\n"..
			GRT_USAGE5.."\n"..
			GRT_USAGE6.."\n"..
			GRT_USAGE7
		);
		return;
	end
	
	-- Split argument into arg1 and arg2 separated by space character
	local arg1, arg2 = GRT_SplitSpace(argument);
		
	-- Toggle the display of the frame
	if (arg1 == "display") then
	
		-- Allow display to be specify on or off, otherwise toggle
		local display = "no";
		if ( arg2 and arg2 == "on") then
			display = "on";
		elseif ( arg2 and arg2 == "off") then
			display = "off";
		elseif ( Gendi.GRT.Visible) then
			display = "off";
		else
			display = "on";
		end
		
		if ( display == "off") then
			GRT_Chat(GRT_HIDDEN);
			-- Lock frame if unlocked
			-- Hide frame
			getglobal("GRTFrame"):Hide();
			-- Set not visible variable
			Gendi.GRT.Visible = false;
			GRT_OptionsFrameEnableCheckButton:SetChecked(0);
		elseif ( display == "on") then
			GRT_Chat(GRT_DISPLAYED);
			-- Show frame
			getglobal("GRTFrame"):Show();
			-- Set visible variable
			Gendi.GRT.Visible = true;
			-- Setup display again to force refresh of display
			GRT_SetupSkillDisplayList();
			GRT_OptionsFrameEnableCheckButton:SetChecked(1);
		end
	-- Set UI scale
	elseif (arg1 == "scale") then
		-- get scale from arg2
		local scale = tonumber(arg2);
		-- if scale is too small or too large then reject
		if (not scale or scale > 1.5 or scale < 0.5 ) then
			GRT_Chat(GRT_BADSCALE);
		else
			-- else set the scale to value stated (note Gendi.GRT.Scale is a saved variable)
			-- Saved scale
			Gendi.GRT.Scale = scale
			-- Set scale
			getglobal("GRTFrame"):SetScale(Gendi.GRT.Scale);
			-- Inform user of new scale
			GRT_Chat(GRT_SCALESET..Gendi.GRT.Scale);
		end
	elseif (arg1 == "options" or arg1 == "set" or arg1 == "config" ) then
		-- Show options window	
		GRT_OptionsFrameList_Update();
		getglobal("GRT_OptionsFrame"):Show();
	elseif (arg1 == "reset" and arg2 == "defaults" ) then
		-- Reset skills to defaults
		GRT_Restore_Defaults();
		GRT_SetupSkillDisplayList();
		GRT_Chat("Defaults restored");
	elseif (arg1 == "lock" ) then
		-- Prevent frame movement
		Gendi.GRT.Locked = true;
		GRT_Chat("Movement locked");
	elseif (arg1 == "unlock" ) then
		-- Allow frame movement
		Gendi.GRT.Locked = false;
		GRT_Chat("Movement unlock");
	elseif (arg1 == "width" ) then
		if (arg2 == "small") then
			Gendi.GRT.Width = 75;
			GRT_Chat("Small skill width");
			GRT_SetupSkillDisplayList();
		elseif (arg2 == "normal") then
			Gendi.GRT.Width = 105;
			GRT_Chat("Normal skill width");
			GRT_SetupSkillDisplayList();
		elseif (arg2 == "large") then
			Gendi.GRT.Width = 135;
			GRT_Chat("Large skill width");
			GRT_SetupSkillDisplayList();
		else
			GRTCommand("");
		end
		
	elseif (arg1 == "icons") then
		if (not Sea) then
			GRT_Chat("Sea is required for icon display");
		elseif ( arg2 == "on") then
			if ( not Gendi.GRT.icons) then
				Gendi.GRT.icons = true;
				GRT_OptionsFrameActionBarEnableCheckButton:SetChecked(1);
			end
			GRT_Chat("Icons on");
		elseif ( arg2 == "off") then
			if ( Gendi.GRT.icons) then
				Gendi.GRT.icons = false;
				GRT_OptionsFrameActionBarEnableCheckButton:SetChecked(0);
			end
			GRT_Chat("Icons off");
		elseif ( arg2 == "") then
			Gendi.GRT.icons = not Gendi.GRT.icons;
			if ( Gendi.GRT.icons) then
				GRT_Chat("Icons on");
				GRT_OptionsFrameActionBarEnableCheckButton:SetChecked(1);
			else
				GRT_Chat("Icons off");
				GRT_OptionsFrameActionBarEnableCheckButton:SetChecked(0);
			end
		else
			GRT_Command("");
		end

	elseif (arg1 == "window") then
		if ( arg2 == "on") then
			if ( not Gendi.GRT.window) then
				Gendi.GRT.window = true;
				GRT_OptionsFrameWindowEnableCheckButton:SetChecked(1);
				GRTFrame:Show();
				GRT_SetupSkillDisplayList();
			end
			GRT_Chat("Window on");
		elseif ( arg2 == "off") then
			if ( Gendi.GRT.window) then
				Gendi.GRT.window = false;
				GRT_OptionsFrameWindowEnableCheckButton:SetChecked(0);
				GRTFrame:Hide();
			end
			GRT_Chat("Window off");
		elseif ( arg2 == "") then
			Gendi.GRT.window = not Gendi.GRT.window;
			if ( Gendi.GRT.window) then
				GRT_Chat("Window on");
				GRT_OptionsFrameWindowEnableCheckButton:SetChecked(1);
				GRTFrame:Show();
				GRT_SetupSkillDisplayList();
			else
				GRT_Chat("Window off");
				GRT_OptionsFrameWindowEnableCheckButton:SetChecked(0);
				GRTFrame:Hide();
			end
		else
			GRT_Command("");
		end

	else
		-- If no recognised syntax then send usage message
		GRTCommand("");
	end
end


-----------------------------
-- Restores default skill list
-----------------------------
function GRT_Restore_Defaults()
	Gendi.GRT = {};
	Gendi.GRT.Version = GRT_VERSION;
	Gendi.GRT.Visible = true;
	Gendi.GRT.Scale = 1.0;
	Gendi.GRT.Width = 105.0;
	Gendi.GRT.Locked = false;
	Gendi.GRT.icons = false;
	Gendi.GRT.window = true;
	Gendi.GRT.FullSkillList = {};
	Gendi.GRT.FullSkillList = GRH_SkillListForTicker;
	
	-- Reset scale and position
	local mainFrame = getglobal("GRTFrame");
	mainFrame:SetScale(Gendi.GRT.Scale);
	--mainFrame:SetUserPlaced(false);
	mainFrame:ClearAllPoints();
	mainFrame:SetPoint("CENTER", "UIParent", "CENTER", 0, 0);
	GRT_OptionsFrameEnableCheckButton:SetChecked(1);
end
-----------------------------
-- Calculates how long you need to wait before enough energy is ready for rogue skills
-----------------------------
function GRT_SkillTickChecker()
	-- Happens when energy changes
	-- Using GRT_LastEnergy for current energy tick rate
	-- Using GRT_CurrEnergy for current energy (could call function again, but why bother when already known)
	
	-- Player's current energy
	local curr = UnitMana("player");
	
	local id = this:GetID();
	-- Get current skill based of button's frame ID (this is the same as the skill's position in list GRT_CurrentSkill)
	local skillInfo = GRT_CurrentSkill[id];
	local button = getglobal("GRTButton"..id);
	local ticker = getglobal("GRTTicker"..id);
	local cooldownstart, cooldown = GetSpellCooldown(skillInfo.spellId, "spell");
	if ( cooldown ~= 0) then
		GRT_Debug("Spell ID: "..id..", Spell "..skillInfo.spellName..",Cooldown "..cooldown);
	end
	-- Set default rgb and alpha values, these are used at the end in setting background colour
	-- Only set after this if they differ from default
	local colour = {{0},{0},{0}}
	local alpha = 0.5;
	local tickerText = "";
	local iconDisplay = 0;
	
	
	if ( not cooldown) then
		cooldown = 0;
	end

	if ( cooldown > 1) then
		-- Spell is in cooldown
		skillInfo.ready = false;		
		skillInfo.cooldown = ceil(( cooldownstart + cooldown ) - GetTime())
		colour[1] = 1;
		tickerText = "CD: "..skillInfo.cooldown.."s";
		iconDisplay = skillInfo.cooldown;
	
	else
		-- Only if there is no cooldown on skill (or cooldown is 1 second or less)
		-- set the skill to have no cooldown
		skillInfo.cooldown = 0;
		
		-- if energy decreases (skill use or drain) then ignore except to reset last know energy to current
		-- if energy increases then reset the tick rate to the difference between last know energy and current
		if ( GRT_CurrEnergy and curr and curr > GRT_CurrEnergy  ) then
			GRT_LastEnergy = curr-GRT_CurrEnergy;
			if ( not GRT_LastEnergy) then
				GRT_LastEnergy = 1;
			end
		end
		
		-- Set last known current energy to current energy
		GRT_CurrEnergy = curr;
		
		-- Show the button in question
		button:Show();
		ticker:Show();
		
		-- Display texture and desciption of skill
		getglobal(button:GetName().."Icon"):SetTexture(skillInfo.texture);
		getglobal(button:GetName().."DescribeText"):SetText(skillInfo.spellName);
		
		local y = 0;
		-- energy is energy required to use skill
		local energy = skillInfo.energy;
		
		
		-- If player has enough energy for this skill already
		if ( energy <= GRT_CurrEnergy) then
			-- Display that the spell is ready
			tickerText = "READY";
			iconDisplay = 0;
			
			-- Set background display to green
			colour[2] = 1;
			
			-- if the skill is a finishing move then change the alpha (how visible the background is)
			-- depending on how many combo points the player has
			-- Note, combo points gained from actions that do not use energy, like dodging with certain talents
			-- are not updated until next energy change.
			if ( skillInfo.stype == "finish") then
				local comboPoints = GetComboPoints();
				
				alpha = comboPoints * 0.1;
				
				-- One difference. Where player has the full 5 combo points (and remember the skill is ready)
				-- then highlight the finishing in a light grey/white with a fairly solid background
				if (comboPoints == 5) then
					colour[1] = 1;
					colour[3] = 1;
					alpha = 1;
				end
			end
			
			-- Set the ready value to true (used when clicking button, click only works if ready is true)
			skillInfo.ready = true;
		else
			-- otherwise the player does not have enough energy
			
			-- Find out how much energy is required (require energy for skill take away current energy left)
			energy = energy - GRT_CurrEnergy;
			
			-- loop taking energy gained per tick from energy still needed and count how many times the tick would need to take place
			local x = 0;
			while energy > 0 do
				-- take tick rate from energy until less than zero and count how many times it will take
				energy = energy - GRT_LastEnergy;
				x = x + 1;
			end
			
			-- Work out background colour based on how many ticks will need to take place
			-- If more than 2 ticks, set colour to red
			if (x > 2) then
				colour[1] = 1;
			-- If 2 tick, set colour to orange
			elseif ( x == 2) then
				colour[1] = 1;
				colour[2] = 0.5;
			-- otherwise just 1 tick is needed, so set colour to yellow
			else
				colour[1] = 1;
				colour[2] = 1;
			end
			
			-- Update the display with number of ticks need
			
			-- plural check
			local ticks=" "..GRT_TICK_PURAL;
			if ( x == 1) then
				ticks=" "..GRT_TICK_SINGULAR;
			end
			tickerText = x..ticks;
			iconDisplay = x;
			-- state that the skill is not ready and hence the button will not fire skill when clicked
			skillInfo.ready = false;
		end
	end
	
	-- Set the background colour of button based on rgb and alpha values calculated above
	GRTSetColor(button, colour, alpha);
	GRTSetColor(ticker, colour, alpha);
	getglobal(ticker:GetName().."TickerText"):SetText(tickerText);
	
	GRT_UpdateIcon(skillInfo, iconDisplay, colour);

end


-----------------------------
-- Sets the background colour of a button based off rgb and alpha values
-----------------------------
function GRTSetColor(button, colour, alpha)
	button:SetBackdropColor(colour[1], colour[2], colour[3], alpha);
end


-----------------------------
-- Sort skills alphabetically by type first and then by name
-----------------------------
function GRT_SortSkills(List, Count)
-- Its a basic sort function, can't be bothered to comment this except to say it sorts the list twice
-- First by name, then by type. This means they will be displayed in type order and then in alphabetical order within types.
    local temp_skill = {};
    local changed = true;

    if Count>1 then
			
		while changed do
			changed = false;
			for i=1,Count-1 do
				if (List[i].spellName > List[i+1].spellName) then
					temp_skill = List[i];
					List[i] = List[i+1];
					List[i+1] = temp_skill;
					changed = true;
				end
			end
		end
		
		changed = true;
		
		while changed do
			changed = false;
			for i=1,Count-1 do
				if (List[i].stype == "finish" and List[i+1].stype == "instant") then
					temp_skill = List[i];
					List[i] = List[i+1];
					List[i+1] = temp_skill;
					changed = true;
				end
			end
		end
    end
end


-----------------------------
-- OnLoad functions
-----------------------------

-----------------------------
-- On load for main frame (this is the part you can move around)
-----------------------------
function GRTFrameOnLoad()

	GRT_Debug("GRTFramOnLoad");
	
	-- setup /grt slash command to call GRTCommand
	-- This is still setup for non rogues incase the frame gets displayed by accident and /grt display can be used to remove it
	GRT_Debug("Setup slash command");
	SlashCmdList["GRT"] = GRTCommand;
	SLASH_GRT1 = "/grt";
	
	-- Get player class (in english) and check if player is rogue.
	GRT_Debug("Checking for class");
	local _, englishClass = UnitClass("player");
	if ( englishClass ~= "ROGUE") then
		-- Not playing rogue, don't function
		this:Hide();
		return;
	end;
	
	GRT_Debug("Player is rogue");
	-- Player is rogue so show main frame
	this:Show();
	
	-- Register Events
	this:RegisterEvent("VARIABLES_LOADED");
	-- Instead I use PLAYER_ENTERING_WORLD which triggers every time a player enters the game, an instance or hearthstone.
	-- But at this stage enough character/game data is loaded to allow setup functions to work
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
end


-----------------------------
-- On load for each button
-- Register events for each button on load, won't register events if player isn't a rogue
-----------------------------
function GRTButtonOnLoad()

	GRT_Debug("GRTButtonOnLoad");
	
	GRT_Debug("Checking for class");
	-- Stop processing button on load if not a rogue
	local _, englishClass = UnitClass("player");
	if ( englishClass ~= "ROGUE") then
		-- Not playing rogue, don't function
		return;
	end;
	GRT_Debug("Player is rogue");
	
	this:SetBackdropBorderColor(nil, nil, nil, 0);
	
	-- Register events to be tracked by button
	
	-- These may be used later, just here currently as placeholders
	--this:RegisterEvent("UPDATE_SHAPESHIFT_FORMS");
	--this:RegisterEvent("SPELLS_CHANGED");
	--this:RegisterEvent("SPELL_UPDATE_COOLDOWN");
	
	-- Triggers when a unit's energy changes.
	this:RegisterEvent("UNIT_ENERGY");
	-- I no longer use the event VARIABLES_LOADED as not enough game/character data is loaded when this triggers
	--this:RegisterEvent("VARIABLES_LOADED");
	-- Instead I use PLAYER_ENTERING_WORLD which triggers every time a player enters the game, an instance or hearthstone.
	-- But at this stage enough character/game data is loaded to allow setup functions to work
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
end


-----------------------------
-- OnLoad for ticker
-----------------------------
function GRTTickerOnLoad()
	this:SetBackdropBorderColor(nil, nil, nil, 0);
end
-----------------------------


-----------------------------
-- OnEvent functions
-----------------------------

-----------------------------
-- On event for main frame
-----------------------------
function GRTFrameOnEvent (event)

	-- If the player is not a rogue then stop processing
	local _, englishClass = UnitClass("player");
	if ( englishClass ~= "ROGUE") then
		-- Not playing rogue, don't function
		return;
	end;

	if (event == "VARIABLES_LOADED") then
		-- Setup defaults for saved values
		if (not Gendi.GRT.Version or Gendi.GRT.Version ~= GRT_VERSION
		 or not Gendi.GRT.FullSkillList or Gendi.GRT.FullSkillList == {} ) then
			GRT_Restore_Defaults();
		end
		
		-- No icons if Sea not present
		if (not Sea) then
			Gendi.GRT.icons = false;
		end
		
		if (not Gendi.GRT.window) then
			GRT_OptionsFrameWindowEnableCheckButton:SetChecked(0);
			this:Hide();
		else
			GRT_OptionsFrameWindowEnableCheckButton:SetChecked(1);
		end
		
		if (not Gendi.GRT.icons) then
			GRT_OptionsFrameActionBarEnableCheckButton:SetChecked(0);
		else
			GRT_OptionsFrameActionBarEnableCheckButton:SetChecked(1);
		end
		
		
	-- Using PLAYER_ENTERING_WORLD instead of variables loaded and an extra true value (which is false at startup)
	-- So that this only triggers the first time a player enters the world (ie from character selection screen)
	elseif (event == "PLAYER_ENTERING_WORLD" and not GRT_EnteredGame) then
		
		-- Set entered game to true so that this section of code will not trigger again
		GRT_EnteredGame = true;
		
		GRT_Debug("GRTFrame "..event);
		
		-- Checks visible value and displays frame if necessary, has default processed on load may be different from saved value
		if ( Gendi.GRT.Visible) then
			GRT_Debug("Visible");
			-- show main frame
			if ( Gendi.GRT.window) then
				this:Show();
			end
			-- set frame UI scale
			this:SetScale(Gendi.GRT.Scale);
			GRT_OptionsFrameEnableCheckButton:SetChecked(1);
			
			-- Setup display and force refresh of display
			GRT_SetupSkillDisplayList();
		else
			GRT_Debug("Hidden");
			-- Hide main frame
			this:Hide();
			-- Don't bother to setup skill list. This will be setup if the window is set to be displayed later
			GRT_OptionsFrameEnableCheckButton:SetChecked(0);
		end
		
		-- If Sea then hook into the ActionButton_OnUpdate function
		if Sea then
			local x = 1;
			while x <= GRT_ButtonCount do
				GRT_Debug("Hooking actionbutton update");
				Sea.util.hook("ActionButton_OnUpdate","GRT_ActionButton_OnUpdate"..x,"after");
				x = x + 1;
			end
		end
		
	-- need to process:-
	--	on talent change
	--	on skill change
	-- These will cause limited problems, so this section is low priority
	end
end


-----------------------------
-- On event tracker for each button
-----------------------------
function GRTButtonOnEvent(event)

	--GRT_Debug(event);
	
	-- Even though there are no event triggers when not a rogue, still throw non rogues out at start
	local _, englishClass = UnitClass("player");
	if ( englishClass ~= "ROGUE") then
		-- Not playing rogue, don't function
		return;
	end;

	
	-- Get the skill assigned to this button.
	local id = this:GetID();
	local skillInfo = GRT_CurrentSkill[id];
	
	-- If no such skill then hide button and stop processing
	if (skillInfo == nil or not skillInfo.enabled) then
		if (this:IsVisible()) then
			this:Hide();
		end
		return;
	end
	
	-- Make sure button is visible
	if (not this:IsVisible()) then
		this:Show();
	end
	
	if ( event == "UNIT_ENERGY"  and arg1 == "player" ) then
		GRT_Debug("GRTButton Engery changed for player");
		-- Process the energy change if the unit triggering the event is the player
		GRT_SkillTickChecker();
	elseif (event == "PLAYER_ENTERING_WORLD") then
		-- Process nothing on startup atm, anything needed is handled by frame startup
		-- Could have called certain parts from here, but cannot be sure which on event
		-- will trigger first and certain whole frame actions need to take place before
		-- button actions.
		-- hook into the onupdate for action bars
	end
end
-----------------------------


-----------------------------
-- OnUpdate functions
-----------------------------

-----------------------------
-- OnUpdate for buttons
-----------------------------
function GRT_ButtonUpdate()

	-- If the player is not a rogue then stop processing
	local _, englishClass = UnitClass("player");
	if ( englishClass ~= "ROGUE") then
		-- Not playing rogue, don't function
		return;
	end;
	
	-- Only do this every 10 updates
	local id = this:GetID();
	GRT_Timer[id] = GRT_Timer[id] + 1;
	if ( GRT_Timer[id] > 9) then
		-- Reset timer
		GRT_Timer[id] = 0;
		-- Get button's skill position in table
		-- Get skill info based off id
		local skillInfo = GRT_CurrentSkill[id];
	
		-- If the skill is currently in cooldown then update the cooldown
		if ( skillInfo.cooldown and skillInfo.cooldown ~= 0) then
			GRT_SkillTickChecker();
		end
	end
end

function GRT_ActionButton_OnUpdate1()
	GRT_ActionButton_OnUpdate(1);
end

function GRT_ActionButton_OnUpdate2()
	GRT_ActionButton_OnUpdate(2);
end

function GRT_ActionButton_OnUpdate3()
	GRT_ActionButton_OnUpdate(3);
end

function GRT_ActionButton_OnUpdate4()
	GRT_ActionButton_OnUpdate(4);
end

function GRT_ActionButton_OnUpdate5()
	GRT_ActionButton_OnUpdate(5);
end

function GRT_ActionButton_OnUpdate6()
	GRT_ActionButton_OnUpdate(6);
end

function GRT_ActionButton_OnUpdate7()
	GRT_ActionButton_OnUpdate(7);
end

function GRT_ActionButton_OnUpdate8()
	GRT_ActionButton_OnUpdate(8);
end

function GRT_ActionButton_OnUpdate9()
	GRT_ActionButton_OnUpdate(9);
end

function GRT_ActionButton_OnUpdate10()
	GRT_ActionButton_OnUpdate(10);
end

function GRT_ActionButton_OnUpdate11()
	GRT_ActionButton_OnUpdate(11);
end

function GRT_ActionButton_OnUpdate12()
	GRT_ActionButton_OnUpdate(12);
end

function GRT_ActionButton_OnUpdate(buttonId)

	-- If the player is not a rogue then stop processing
	local _, englishClass = UnitClass("player");
	if ( englishClass ~= "ROGUE") then
		-- Not playing rogue, don't function
		return;
	end;
	
	-- No button, ditch
	if ( not this) then
		return;
	end

	local skillInfo = GRT_CurrentSkill[buttonId];
	
	-- If there is not a skill or the skill is off then stop
	if ( not skillInfo or not skillInfo.enabled) then
		return;
	end
	
	local slot = ActionButton_GetPagedID(this);
	local texture = GetActionTexture(slot);
	
	-- if this button is the same as the skills button
	if ( skillInfo.button and skillInfo.button == this) then
		if ( texture ~= skillInfo.texture) then
			GRT_Debug(skillInfo.texture.." no longer matches slot "..slot);
			skillInfo.button = nil;
		end
	else
		if ( texture == skillInfo.texture) then
			GRT_Debug(texture.." texture matches for slot "..slot);
			skillInfo.button = this;
		end
	end

end
-----------------------------

--[[
-----------------------------
-- OnClick functions
-----------------------------

-----------------------------
-- OnClick for buttons
-----------------------------
function GRT_ButtonOnClick()
	-- Find skill for button
	local skill = GRT_CurrentSkill[this:GetID()];
	GRT_Debug("Button pressed");
	
	-- even though should not be visible and clickable if no skill, still check for this
	if ( skill == nil) then
		GRT_Debug("Visible button with no skill detected");
		return;
	end
	
	GRT_Debug("Skill :"..skill.spellName);
	
	-- If skill is not ready (not enough energy to use) then do not process click
	-- Saves a click processing time
	if ( not skill.ready) then
	GRT_Debug("Skill not ready");
		return;
	end
	
	-- Use skill
	-- 1.7 remove
	CastSpellByName(skill.spellName.."("..skill.rank..")");
	-- As of patch 1.7:-
	--"If you omit the rank of a spell in /cast or CastSpellByName(), it will use the highest rank of the spell which you know."
	-- So I will be calling
	-- 1.7 CastSpellByName(skill.spellName);
	-- and removing all trace of rank setup, which would mean less processing at startup if only the spell texture could be found from name instead of Id
end
-----------------------------
]]

-----------------------------
-- Functions for options window
-----------------------------

-----------------------------
-- Option close button pressed
-----------------------------
function GRT_OptionsFrameCloseButton_OnClick()

    getglobal("GRT_OptionsFrame"):Hide();
	-- Reprocess skill list to check for changes (maybe better than doing for every click, maybe not, but this way for now)
	GRT_SetupSkillDisplayList();
end


-----------------------------
-- Moving frame
-----------------------------
function GRT_MouseDown()
	if ( not Gendi.GRT.Locked) then
		this:StartMoving();
	end
end

function GRT_MouseUp()
	if ( not Gendi.GRT.Locked) then
		this:StopMovingOrSizing();
	end
end
-----------------------------


-----------------------------
-- Toggle skill display when clicked (only changes real display on exiting options screen(
-----------------------------
function GRT_OptionsFrameNameButton_OnClick(arg1)

    local index;

	-- Get real index based of scroll window
    index = arg1 + FauxScrollFrame_GetOffset(GRT_OptionsFrameScrollFrame)

	-- If valid skill for clicked location
	if (Gendi.GRT.FullSkillList[index]) then
		-- Toggle enabled
		Gendi.GRT.FullSkillList[index].enabled = not Gendi.GRT.FullSkillList[index].enabled;
		GRT_OptionsFrameList_Update();
	end
end


-----------------------------
-- Display tooltip for spell mouseover
-----------------------------
function GRT_OptionsFrameNameButton_OnEnter()
	GameTooltip_SetDefaultAnchor(GameTooltip,this);
    GameTooltip:SetText(GRT_TOOLTIP_NAME1);
    GameTooltip:AddLine(GRT_TOOLTIP_NAME2, .75, .75, .75, 1);
    GameTooltip:Show();
end


-----------------------------
-- Display tooltip for enabled checkbox
-----------------------------
function GRT_OptionsFrameCheckButton_OnEnter()
	GameTooltip_SetDefaultAnchor(GameTooltip,this);
    GameTooltip:SetText(GRT_TOOLTIP_ENABLE1);
    GameTooltip:AddLine(GRT_TOOLTIP_ENABLE2, .75, .75, .75, 1);
    GameTooltip:Show();
end


-----------------------------
-- Display tooltip for Action checkbox
-----------------------------
function GRT_OptionsFrameActionBar_OnEnter()
	GameTooltip_SetDefaultAnchor(GameTooltip,this);
    GameTooltip:SetText(GRT_TOOLTIP_ACTIONENABLE1);
    GameTooltip:AddLine(GRT_TOOLTIP_ACTIONENABLE2, .75, .75, .75, 1);
    GameTooltip:Show();
end


-----------------------------
-- Display tooltip for Window checkbox
-----------------------------
function GRT_OptionsFrameWindow_OnEnter()
	GameTooltip_SetDefaultAnchor(GameTooltip,this);
    GameTooltip:SetText(GRT_TOOLTIP_WINDOWENABLE1);
    GameTooltip:AddLine(GRT_TOOLTIP_WINDOWENABLE2, .75, .75, .75, 1);
    GameTooltip:Show();
end


-----------------------------
-- deal with checkbox toggle for enable
-----------------------------
function GRT_OptionsFrameEnableCheckButton_OnClick()
	-- cheat and just call /grt display xx
	-- if checked then display
	if (this:GetChecked()==1) then
		GRTCommand("display on");
	else
		-- else hide
		GRTCommand("display off");
	end
end


-----------------------------
-- deal with checkbox toggle for action bar
-----------------------------
function GRT_OptionsFrameActionBar_OnClick()
	-- cheat and just call /grt icons to toggle
	-- if checked then display
	GRTCommand("icons");
	--[[
	if (this:GetChecked()==1) then
		GRTCommand("icons on");
	else
		-- else hide
		GRTCommand("icons off");
	end
	]]
end


-----------------------------
-- deal with checkbox toggle for window
-----------------------------
function GRT_OptionsFrameWindow_OnClick()
	-- cheat and just call /grt window xx
	-- if checked then display
	if (this:GetChecked()==1) then
		GRTCommand("window on");
	else
		-- else hide
		GRTCommand("window off");
	end
end


-----------------------------
-- Setup display of list
-----------------------------
function GRT_OptionsFrameList_Update()

    local index;
    local i;
    local itemText;
    local itemButton;
	local scrollSize = 8;

    FauxScrollFrame_Update(GRT_OptionsFrameScrollFrame, GRT_TOTALSKILLCOUNT, scrollSize, scrollSize);

    for i=1,scrollSize do
		index = i + FauxScrollFrame_GetOffset(GRT_OptionsFrameScrollFrame);
        itemButton = getglobal("GRT_OptionsFrameName"..i);
		itemText = getglobal("GRT_OptionsFrameName"..i.."_Text");
		if ( Gendi.GRT.FullSkillList[index]) then
			name = Gendi.GRT.FullSkillList[index].spellName;
		else
			name = " ";
		end
			
		
		if ( index <= GRT_TOTALSKILLCOUNT) then
			itemText:SetText(name);
			itemText:Show();
			if ( Gendi.GRT.FullSkillList[index].enabled) then
				itemButton:LockHighlight();
			else
				itemButton:UnlockHighlight();
			end
		else
			itemText:Hide();
			itemButton:UnlockHighlight();
		end
    end
end



-- Updates the icon for the skill is it exists
function GRT_UpdateIcon(skill, number, colour)
	
	GRT_Debug("Update icon: "..skill.spellName);
	-- if no icon setup for that skill then stop
	if ( not skill.button) then
		GRT_Debug("no button");
		return;
	end
	
	-- if icon use is disabled then stop
	if ( not Gendi.GRT.icons) then
		GRT_Debug("icons off");
		return;
	end
		
	local slot = ActionButton_GetPagedID(skill.button);
	local buttonName = skill.button:GetName()
	GRT_Debug(buttonName);
	GRT_Debug(slot);
	local normalTexture = getglobal(buttonName.."NormalTexture");
	local slotText = getglobal(buttonName.."Name");
	
	GRT_Debug("action slot"..slot);
	local texture = GetActionTexture(slot);
	local count = "";
	
	if (texture ~= skill.texture) then
		-- icon became something else, remove icon from skill, display will be updated with blank
		skill.button = nil;
	elseif ( number ~= 0) then
		-- else if the number to display isn't 0, when 0 display will be updated with blank
		count = number;
	end
			
	slotText:SetText(count);
	local height = skill.button:GetHeight();
	slotText:SetFont("Fonts\\FRIZQT__.TTF", height);
	slotText:SetTextColor(colour[1], colour[2], colour[3]);
end
